import math
import numpy
import matplotlib
import matplotlib.pyplot

import shared
import atomic_data
import functions


###=====================================================================================================================
def Gaussian(x, mu, sigma):
    return numpy.exp(-numpy.power((x-mu)/sigma, 2.0) / 2) / sigma / math.sqrt(2.0 * math.pi)
#

###=====================================================================================================================
def RDF_Like_Fingerprint(atomid, rpos):
    if (shared.n_r_array is None):
        r_max = math.ceil(shared.a * math.sqrt(3.0) / shared.fingerprint_delta) * shared.fingerprint_delta
        shared.r_array = numpy.arange(0.0, r_max + shared.fingerprint_delta / 2, shared.fingerprint_delta)
        shared.n_r_array = len(shared.r_array)
    #

    factor = shared.a ** 3 / (4 * math.pi ** 2 * shared.fingerprint_delta * shared.fingerprint_sigma * math.sqrt(2.0))

    unique_atomid = numpy.unique(atomid)
    n_unique_atomid = len(unique_atomid)

    fp_section_key = []
    fp_section_val = []
    for i1_unique_atomid in range(0, n_unique_atomid):
        for i2_unique_atomid in range(i1_unique_atomid, n_unique_atomid):
            fp_section_key.append((unique_atomid[i1_unique_atomid], unique_atomid[i2_unique_atomid]))

            s = numpy.zeros(shared.n_r_array)
            n1_atom = 0
            n2_atom = 0
            for i_atom in numpy.nonzero(atomid == unique_atomid[i1_unique_atomid])[0]:
                n1_atom = n1_atom + 1
                for j_atom in numpy.nonzero(atomid == unique_atomid[i2_unique_atomid])[0]:
                    n2_atom = n2_atom + 1
                    r_ij_vec = rpos[j_atom,:] - rpos[i_atom,:]
                    r_ij = numpy.linalg.norm(r_ij_vec) * shared.a
                    if (r_ij > 1.0e-9):
                        s = s + numpy.exp(-0.5 * ((r_ij - shared.r_array) / shared.fingerprint_sigma) ** 2)
                    #
                #
            #
            fp_section_val.append(s * factor / (n1_atom * n2_atom) - 1)
    #

    return (fp_section_key, fp_section_val)
#

###=====================================================================================================================
def Save_Fingerprint(fp_val, filename):
    data = numpy.vstack((shared.r_array, fp_val[0]))
    for i in range(1, len(fp_val)):
        data = numpy.vstack((data, fp_val[i]))
    #
    numpy.savetxt(filename, numpy.transpose(data), fmt='%24.15e')
#

###=====================================================================================================================
def Cosine_Distance(fp_val1, fp_val2):
    tmp_val0 = 0.0
    tmp_val1 = 0.0
    tmp_val2 = 0.0
    for i_section in range(0, len(fp_val1)):
        tmp_val0 = tmp_val0 + numpy.inner(fp_val1[i_section], fp_val2[i_section])
        tmp_val1 = tmp_val1 + numpy.inner(fp_val1[i_section], fp_val1[i_section])
        tmp_val2 = tmp_val2 + numpy.inner(fp_val2[i_section], fp_val2[i_section])
    #
    if ((tmp_val1 < 1.0e-9) or (tmp_val2 < 1.0e-9)):
        print(tmp_val0, tmp_val1, tmp_val2)
        Save_Fingerprint(fp_val1, "1.fpt")
        Save_Fingerprint(fp_val2, "2.fpt")
        raise
        exit()
    #

    return (1.0 - tmp_val0 / math.sqrt(tmp_val1 * tmp_val2)) / 2
#

###=====================================================================================================================
def Is_Unique(fp_val):
    for i_fp in range(0, len(shared.fingerprint_archive)):
        fp_dist = Cosine_Distance(fp_val, shared.fingerprint_archive[i_fp])
        if (fp_dist < shared.fingerprint_threshold):
            return False
        #
    #

    return True
#

###=====================================================================================================================
def Plot_Fingerprint(fp_section_key, fp_section_val, plotfile):
    y_max = numpy.amax(numpy.concatenate((fp_section_val)))

    r_max = shared.r_array[-1]

    n_subplot = len(fp_section_key)
    figure_width = max(n_subplot*6, 8)

    left_margin_width = 0.070
    plot_width = 0.900
    subplot_width = plot_width / n_subplot

    figure_size_unit = 1.0
    font_size_unit = 1.0
    pad_size_unit = 1.0

    figure = matplotlib.pyplot.figure(num=None, figsize=(figure_width*figure_size_unit,6*figure_size_unit), dpi=300, facecolor='w')

    subplot = []
    for i_subplot in range(0, n_subplot):
        p = [left_margin_width+i_subplot*subplot_width, 0.160, subplot_width, 0.800]
        subplot.append(figure.add_subplot(n_subplot,1,i_subplot+1, position=p, frame_on=True, zorder=110))

        # draw a black box to hide the grid lines that cover the main frame
        fancybox = matplotlib.patches.FancyBboxPatch([0,0], 1, 1, transform=subplot[-1].transAxes,
                                                     boxstyle='round,pad=0.0', linestyle='solid', linewidth=2.0,
                                                     edgecolor='black', facecolor='none', zorder=221)
        subplot[-1].add_patch(fancybox)

        subplot[-1].plot(shared.r_array, fp_section_val[i_subplot], zorder=201, linestyle='-', linewidth=1, color='red', marker='', label="")


        # label
        box_x = 0.30
        box_y = 0.90
        box_w = 0.40
        box_h = 0.05
        font_scale = 20
        fancybox = matplotlib.patches.FancyBboxPatch([box_x, box_y], box_w, box_h, transform=subplot[-1].transAxes,
                                          boxstyle='round,pad=0.01', linestyle='solid', linewidth=2.0,
                                          edgecolor='black', facecolor='black', zorder=121)
        subplot[-1].add_patch(fancybox)
        symbol1 = atomic_data.atom_list[shared.uniq_atom_numb[fp_section_key[i_subplot][0]]].symb
        symbol2 = atomic_data.atom_list[shared.uniq_atom_numb[fp_section_key[i_subplot][1]]].symb
        subplot[-1].text(box_x+box_w/2, box_y+box_h/2-0.00026*font_scale*font_size_unit,
                         "%s-%s"%(symbol1,symbol2), transform=subplot[-1].transAxes,
                         size=font_scale*font_size_unit, rotation=0.0, ha='center', va='center', color='white', zorder=122)


        ###=============================================================================================================
        frame = subplot[-1].patch
        frame.set_linewidth(2.00)
        frame.set_edgecolor('black')

        major_tick_width = 2.00
        minor_tick_width = 1.00
        major_tick_length = 10.00
        minor_tick_length =  5.00
        major_grid_width = 2.00
        minor_grid_width = 1.00
        major_grid_color = '#E0E0E0'
        minor_grid_color = '#E0E0E0'


        ###-------------------------------------------------------------------------------------------------------------
        x_min_view = 0.0
        x_max_view = math.ceil(r_max)
        tickstep = 1
        axis_label = "R [Angstrom]"

        (major_tick_locations, minor_tick_locations) = functions.Linear_Tick_Calculator(x_min_view, x_max_view, tickorigin=0.0, tickstep=tickstep, nminortick=0)
        n_tick = len(major_tick_locations)
        ticklabelstep = round(n_tick / 6)
        tmp_major_tick_locations = list(major_tick_locations[::ticklabelstep])
        major_tick_labels = [""] * n_tick
        if (x_max_view - tmp_major_tick_locations[-1] < 1):
            del tmp_major_tick_locations[-1]
            major_tick_labels[::ticklabelstep] = ["%i" % x for x in tmp_major_tick_locations] + [""]
        else:
            major_tick_labels[::ticklabelstep] = ["%i" % x for x in tmp_major_tick_locations]
        #
        if (i_subplot != 0):
            major_tick_labels[0] = ""
        #

        subplot[-1].xaxis.set_minor_locator(matplotlib.ticker.FixedLocator(minor_tick_locations))
        subplot[-1].xaxis.set_major_locator(matplotlib.ticker.FixedLocator(major_tick_locations))
        subplot[-1].xaxis.set_ticklabels(major_tick_labels, minor=False)
        subplot[-1].xaxis.set_tick_params(which='minor', direction='in', bottom=True, top=True, left=True, right=True, length=minor_tick_length, width=minor_tick_width, zorder=101)
        subplot[-1].xaxis.set_tick_params(which='major', direction='in', bottom=True, top=True, left=True, right=True, length=major_tick_length, width=major_tick_width, pad=7*pad_size_unit, labelsize=18*font_size_unit, zorder=102)
        subplot[-1].xaxis.set_label_text(axis_label, fontsize=28*font_size_unit, verticalalignment='top')
        subplot[-1].xaxis.labelpad = 7 * pad_size_unit
        subplot[-1].xaxis.grid(which='minor', color=minor_grid_color, linestyle='-', linewidth=minor_grid_width, zorder=101)
        subplot[-1].xaxis.grid(which='major', color=major_grid_color, linestyle='-', linewidth=major_grid_width, zorder=102)
        subplot[-1].xaxis.set_view_interval(x_min_view, x_max_view)
        subplot[-1].set_xlim(left=x_min_view, right=x_max_view)


        ###-------------------------------------------------------------------------------------------------------------
        tickstep = y_max / 8
        n_digits = math.floor(math.log10(abs(tickstep)))
        tickstep = round(tickstep, -n_digits)
        nminortick = 4
        y_min_view = -2.0
        y_max_view = y_max * 1.05

        (major_tick_locations, minor_tick_locations) = functions.Linear_Tick_Calculator(y_min_view, y_max_view, tickorigin=0.0, tickstep=tickstep, nminortick=nminortick)

        subplot[-1].yaxis.set_minor_locator(matplotlib.ticker.FixedLocator(minor_tick_locations))
        subplot[-1].yaxis.set_major_locator(matplotlib.ticker.FixedLocator(major_tick_locations))

        if (i_subplot == 0):
            axis_label = "Fingerprint"
            major_tick_labels = ["%i" % x for x in major_tick_locations]

            subplot[-1].yaxis.set_ticklabels(major_tick_labels, minor=False)
            subplot[-1].yaxis.set_tick_params(which='minor', direction='in', bottom=True, top=True, left=True, right=True, length=minor_tick_length, width=minor_tick_width, zorder=101)
            subplot[-1].yaxis.set_tick_params(which='major', direction='in', bottom=True, top=True, left=True, right=True, length=major_tick_length, width=major_tick_width, pad=5*pad_size_unit, labelsize=18*font_size_unit, zorder=102)
            subplot[-1].yaxis.set_label_text(axis_label, fontsize=24*font_size_unit, verticalalignment='center')
            subplot[-1].yaxis.labelpad = 26 * pad_size_unit
            subplot[-1].yaxis.grid(which='minor', color=minor_grid_color, linestyle='-', linewidth=minor_grid_width, zorder=101)
            subplot[-1].yaxis.grid(which='major', color=major_grid_color, linestyle='-', linewidth=major_grid_width, zorder=102)
            subplot[-1].yaxis.set_view_interval(y_min_view, y_max_view)
            subplot[-1].set_ylim(bottom=y_min_view, top=y_max_view)
        else:
            axis_label = ""
            major_tick_labels = []

            subplot[-1].yaxis.set_ticklabels(major_tick_labels, minor=False)
            subplot[-1].yaxis.set_tick_params(which='minor', direction='in', bottom=True, top=True, left=True, right=True, length=minor_tick_length, width=minor_tick_width, zorder=101)
            subplot[-1].yaxis.set_tick_params(which='major', direction='in', bottom=True, top=True, left=True, right=True, length=major_tick_length, width=major_tick_width, pad=5*pad_size_unit, labelsize=18*font_size_unit, zorder=102)
            subplot[-1].yaxis.set_label_text(axis_label, fontsize=24*font_size_unit, verticalalignment='center')
            subplot[-1].yaxis.labelpad = 26 * pad_size_unit
            subplot[-1].yaxis.grid(which='minor', color=minor_grid_color, linestyle='-', linewidth=minor_grid_width, zorder=101)
            subplot[-1].yaxis.grid(which='major', color=major_grid_color, linestyle='-', linewidth=major_grid_width, zorder=102)
            subplot[-1].yaxis.set_view_interval(y_min_view, y_max_view)
            subplot[-1].set_ylim(bottom=y_min_view, top=y_max_view)


    ###=================================================================================================================
    matplotlib.pyplot.savefig(plotfile)

    matplotlib.pyplot.close(figure)
#

########################################################################################################################

