import numpy
import scipy
import os
import operator
import signal

import shared
import potentials
import fingerprint
import save
import atomic_data


###=====================================================================================================================
def signal_handler(signum, frame):
    raise Exception("Timed out!")
#

###=====================================================================================================================
def Evaluate(struct_list):
    for struct in struct_list:
        if (struct.origin != "survival"):
            struct.energy = potentials.Structure_Energy(struct)
            struct.force  = potentials.Structure_Force(struct)
        #
    #
#

###=====================================================================================================================
def Relax(struct_list):
    bounds = [(0.0,1.0)] * shared.n_atom * 3
    with open("fitness.log", 'w') as outf:
        for struct in struct_list:
            if (struct.origin != "survival"):
                print("i_generation, i_indiv, origin: %4i %4i %s ... " % (struct.i_generation+1, struct.i_indiv+1, struct.origin), file=outf, end="")

                if (shared.relax):
                    success = False
                    signal.signal(signal.SIGALRM, signal_handler)
                    signal.alarm(shared.max_relaxation_time)
                    try:
                        #res = scipy.optimize.minimize(potentials.Structure_E, struct.rpos.reshape(-1), method='BFGS', jac=potentials.Structure_Der, options={'disp':False, 'gtol':1e-12, 'eps':1e-12})
                        res = scipy.optimize.minimize(potentials.Structure_E, struct.rpos.reshape(-1), method='L-BFGS-B', jac=potentials.Structure_Der, bounds=bounds, options={'disp':False, 'gtol':1e-12, 'eps':1e-12})
                        #res = scipy.optimize.minimize(potentials.Structure_E, struct.rpos.reshape(-1), method='Newton-CG', jac=potentials.Structure_Der, bounds=bounds, options={'disp':False})
                        success = (res.success and (res.fun <= struct.energy))
                        print("DONE.", file=outf)
                    except Exception:
                        print("Timed out!", file=outf)
                    #
                    signal.alarm(0)

                    struct.relaxed = success
                    if (success):
                        struct.rpos = numpy.reshape(numpy.copy(res.x), (shared.n_atom, 3))
                        struct.energy = potentials.Structure_Energy(struct)
                        struct.force  = potentials.Structure_Force(struct)
                    #
                #
            #
        #
    #
#

###=====================================================================================================================
def Analyse(struct_list):
    tmp_i_struct_list = []
    tmp_energy_list   = []
    for i_struct in range(0, len(struct_list)):
        struct_list[i_struct].Calculate_RDF_Like_Fingerprint()
        tmp_i_struct_list.append(i_struct)
        tmp_energy_list.append(struct_list[i_struct].energy)
        #
    #
    if (len(tmp_energy_list) < len(struct_list) * shared.min_conv_fraction):
        print(":ERROR:   !!!   Too few structures were successfully relaxed: %i" % len(tmp_energy_list))
        raise
        exit()
    #
    n_successful_energy = len(tmp_energy_list)

    tmp_energy_indx_array = numpy.argsort(numpy.array(tmp_energy_list), kind='mergesort')

    for i_indx in range(0, len(tmp_energy_indx_array)):
        energy_indx = tmp_energy_indx_array[i_indx]
        i_struct = tmp_i_struct_list[energy_indx]
        struct_list[i_struct].i_indiv_energy = i_indx
    #

    category = []
    i_indx = 0
    energy_indx = tmp_energy_indx_array[i_indx]
    i_struct = tmp_i_struct_list[energy_indx]
    category.append([i_struct])
    struct_list[i_struct].i_category = 0
    struct_list[i_struct].unique     = True
    for i_indx in range(1, len(tmp_energy_indx_array)):
        energy_indx = tmp_energy_indx_array[i_indx]
        i_struct = tmp_i_struct_list[energy_indx]
        fp_dist_list = []
        for i_category in range(0, len(category)):
            i_struct_head = category[i_category][0]
            fp_dist = fingerprint.Cosine_Distance(struct_list[i_struct].fp_val, struct_list[i_struct_head].fp_val)
            fp_dist_list.append((i_category, fp_dist))
        #
        (i_min, fp_dist_min) = sorted(fp_dist_list, key=operator.itemgetter(1))[0]
        if (fp_dist_min > shared.fingerprint_threshold):
            category.append([i_struct])
            struct_list[i_struct].i_category = len(category) - 1
            struct_list[i_struct].unique     = True
        else:
            category[i_min].append(i_struct)
            struct_list[i_struct].i_category = i_min
            struct_list[i_struct].unique     = False
        #
    #
    shared.energy_array  = numpy.array([struct_list[item[0]].energy  for item in category])
    shared.i_indiv_array = numpy.array([struct_list[item[0]].i_indiv for item in category], dtype=int)
    energy_indx_array    = numpy.argsort(shared.energy_array, kind='mergesort')
    shared.energy_array  = shared.energy_array[energy_indx_array]
    shared.i_indiv_array = shared.i_indiv_array[energy_indx_array]

    shared.n_death     = int(shared.n_indiv * shared.death_fraction)
    shared.n_live      = min(n_successful_energy, shared.n_indiv-shared.n_death)
    shared.n_live_uniq = min(len(shared.energy_array), shared.n_live)
    shared.n_surviving = min(shared.n_live_uniq, int(shared.n_indiv*shared.surviving_fraction))
    if (shared.n_live_uniq > 1):
        shared.n_crossing  = int(shared.n_indiv * shared.crossing_fraction)
    else:
        shared.n_crossing  = 0
    #
    shared.n_slice_mutation  = int(shared.n_indiv * shared.slice_mutation_fraction)
    #
    if (shared.n_uniq_atom > 1):
        shared.n_atom_type_mutation  = int(shared.n_indiv * shared.atom_type_mutation_fraction)
    else:
        shared.n_atom_type_mutation  = 0
    #
    shared.n_atom_position_mutation  = int(shared.n_indiv * shared.atom_position_mutation_fraction)
    #
    shared.n_random    = shared.n_indiv - shared.n_crossing - shared.n_slice_mutation - shared.n_atom_type_mutation - shared.n_atom_position_mutation

    shared.best_energy_indx_array = []
    i_uniq = -1
    for i_indx in range(0, len(tmp_energy_indx_array)):
        energy_indx = tmp_energy_indx_array[i_indx]
        i_struct = tmp_i_struct_list[energy_indx]
        if (struct_list[i_struct].unique):
            i_uniq = i_uniq + 1
            if (i_uniq < shared.n_surviving):
                struct_list[i_struct].status = "surviving"
                shared.best_energy_indx_array.append(i_struct)
            elif (i_uniq < shared.n_live_uniq):
                struct_list[i_struct].status = "live"
            else:
                struct_list[i_struct].status = "dead"
            #
        else:
            struct_list[i_struct].status = "dead"
        #
    #
    shared.best_energy_indx_array = numpy.array(shared.best_energy_indx_array)

    datapath = os.path.join("%s" % (shared.work_path), "generation_%4.4i/data/" % (shared.i_generation + 1))
    if (not os.path.isdir(datapath)):
        os.makedirs(datapath)
    #
    filename = os.path.join(datapath, "data.dat")
    with open(filename, 'w') as outf:
        for i_indx in range(0, len(tmp_energy_indx_array)):
            energy_indx = tmp_energy_indx_array[i_indx]
            i_struct = tmp_i_struct_list[energy_indx]
            if (struct_list[i_struct].unique):
                status = "unique"
            else:
                status = "duplicate"
            #
            history = struct_list[i_struct].history[0].split()
            print("%4i   %4i   %21.9f   %4i   %10s   %10s   %s   %s" % (i_indx+1, struct_list[i_struct].i_indiv+1, struct_list[i_struct].energy, struct_list[i_struct].i_category+1, status, struct_list[i_struct].status, history[0], history[1]), file=outf)
        #
    #
#
###=====================================================================================================================


