###!/usr/bin/env python3
import numpy
import math
import os
import re
import matplotlib
import matplotlib.pyplot

import functions


n_decimals = 1
y_tick_label_format = "%%.%if" % (n_decimals)

work_directory_path = "workspace"
plot_file_path = os.path.join(os.getcwd(), "energy.pdf")

########################################################################################################################
all_dir_list = functions.List_Directories(path=work_directory_path)

generation_dir_list = []
for anydir in all_dir_list:
    if (re.match("^generation_[0-9][0-9][0-9][0-9]$", anydir)):
        generation_dir_list.append(anydir)
#

if (len(generation_dir_list) == 0):
    print(":ERROR:   !!!   No generation directory was found in the work path: '%s'." % (work_directory_path))
    exit(1)
else:
    print("%i generation directories were found in the work path '%s' ..." % (len(generation_dir_list), work_directory_path))
    print("Searching for proper data files in all generation directories ...")
#


data_dict = {}
data_dict["overall"]  = []
data_dict["survival"] = []

for anydir in generation_dir_list:
    data_file = os.path.join(work_directory_path, anydir, "data", "data.dat")
    energy, status = numpy.loadtxt(data_file, usecols=(2,5), dtype=str, unpack=True)

    data_dict["overall"].append(energy.astype(numpy.float))
    data_dict["survival"].append(energy[status == "surviving"].astype(numpy.float))
#

n_generation = len(data_dict["overall"])

###---------------------------------------------------------------------------------------------------------------------
figure_size_unit = 1.0
font_size_unit = 1.0
pad_size_unit = 1.0

figure_width  = max(18, n_generation * 0.6)
#figure_width  = shared.n_indiv_max * figure_size_unit + 6
#figure_height = shared.n_generation * figure_size_unit * 1.5 + 3
#figure = matplotlib.pyplot.figure(num=None, figsize=(figure_width,figure_height), dpi=300, facecolor='w')

hm = 0.300 * 8 / figure_width
w  = 1.000 - 0.350 * 8 / figure_width
#vm = 0.160 * 6 / figure_height
#h  = 1.000 - 0.550 * 6 / figure_height

figure = matplotlib.pyplot.figure(num=None, figsize=(figure_width*figure_size_unit,12*figure_size_unit), dpi=300, facecolor='w')

subplot = {}
subplot["overall"]  = figure.add_subplot(1,2,2, position=[hm, 0.100, w, 0.425], frame_on=True, zorder=110)
subplot["survival"] = figure.add_subplot(1,2,2, position=[hm, 0.535, w, 0.425], frame_on=True, zorder=110)

for subplot_id in subplot.keys():

    fancybox = matplotlib.patches.FancyBboxPatch([0,0], 1, 1, transform=subplot[subplot_id].transAxes,
                                                    boxstyle='round,pad=0.0', linestyle='solid', linewidth=2.0,
                                                    edgecolor='black', facecolor='none', zorder=321)
    subplot[subplot_id].add_patch(fancybox)

    frame = subplot[subplot_id].patch
    frame.set_linewidth(2.00)
    frame.set_edgecolor('black')

    major_tick_width  =  2.00
    minor_tick_width  =  1.00
    major_tick_length = 10.00
    minor_tick_length =  5.00
    major_grid_width  =  2.00
    minor_grid_width  =  1.00
    major_grid_color  = '#E0E0E0'
    minor_grid_color  = '#E0E0E0'

    ###---------------------------------------------------------------------------------------------------------------------
    tickstep   = 1
    nminortick = 0

    x_min_view = 0.0
    x_max_view = n_generation + 1

    major_tick_locations = numpy.arange(x_min_view, x_max_view)
    minor_tick_locations = numpy.array([])

    if (subplot_id == "overall"):
        major_tick_labels = ["%i" % x for x in major_tick_locations]
        major_tick_labels[0] = ""
        axis_label = "Generation"
    else:
        major_tick_labels = []
        axis_label = ""
    #

    subplot[subplot_id].xaxis.set_minor_locator(matplotlib.ticker.FixedLocator(minor_tick_locations))
    subplot[subplot_id].xaxis.set_major_locator(matplotlib.ticker.FixedLocator(major_tick_locations))
    subplot[subplot_id].xaxis.set_ticklabels(major_tick_labels, minor=False)
    subplot[subplot_id].xaxis.set_tick_params(which='minor', direction='in', bottom=True, top=True, left=True, right=True, length=minor_tick_length, width=minor_tick_width, zorder=101)
    subplot[subplot_id].xaxis.set_tick_params(which='major', direction='in', bottom=True, top=True, left=True, right=True, length=major_tick_length, width=major_tick_width, pad=7*pad_size_unit, labelsize=18*font_size_unit, zorder=102)
    subplot[subplot_id].xaxis.set_label_text(axis_label, fontsize=28*font_size_unit, verticalalignment='top')
    subplot[subplot_id].xaxis.labelpad = 7 * pad_size_unit
    subplot[subplot_id].xaxis.grid(which='minor', color=minor_grid_color, linestyle='-', linewidth=minor_grid_width, zorder=101)
    subplot[subplot_id].xaxis.grid(which='major', color=major_grid_color, linestyle='-', linewidth=major_grid_width, zorder=102)
    subplot[subplot_id].set_xlim(left=x_min_view, right=x_max_view)

    ###---------------------------------------------------------------------------------------------------------------------
    y_min = numpy.amin((numpy.concatenate(data_dict[subplot_id][:])))
    y_max = numpy.amax((numpy.concatenate(data_dict[subplot_id][:])))

    delta0 = y_max - y_min
    y_min = y_min - max(0.01*delta0, 1.0)
    y_max = y_max + max(0.01*delta0, 1.0)

    delta  = delta0 / 6
    n = -math.floor(math.log10(delta))
    delta = delta * 10 ** n
    delta = float(round(delta)) / (10 ** n)

    tickstep   = delta
    nminortick = 1
    minortickstep = tickstep / (nminortick + 1)

    (major_tick_locations, minor_tick_locations) = functions.Linear_Tick_Calculator(y_min, y_max, tickorigin=0.0, tickstep=tickstep, nminortick=nminortick, end="minor")
    y_min_view = min(major_tick_locations[0], minor_tick_locations[0])
    y_max_view = max(major_tick_locations[-1], minor_tick_locations[-1])

    major_tick_labels = [y_tick_label_format % x for x in major_tick_locations]
    axis_label = "Energy [Arbitrary unit]"

    subplot[subplot_id].yaxis.set_minor_locator(matplotlib.ticker.FixedLocator(minor_tick_locations))
    subplot[subplot_id].yaxis.set_major_locator(matplotlib.ticker.FixedLocator(major_tick_locations))
    subplot[subplot_id].yaxis.set_ticklabels(major_tick_labels, minor=False)
    subplot[subplot_id].yaxis.set_tick_params(which='minor', direction='in', bottom=True, top=True, left=True, right=True, length=minor_tick_length, width=minor_tick_width, zorder=101)
    subplot[subplot_id].yaxis.set_tick_params(which='major', direction='in', bottom=True, top=True, left=True, right=True, length=major_tick_length, width=major_tick_width, pad=5*pad_size_unit, labelsize=18*font_size_unit, zorder=102)
    subplot[subplot_id].yaxis.set_label_text(axis_label, fontsize=28*font_size_unit, verticalalignment='center')
    subplot[subplot_id].yaxis.labelpad = 28 * pad_size_unit
    subplot[subplot_id].yaxis.grid(which='minor', color=minor_grid_color, linestyle='--', linewidth=minor_grid_width, zorder=101)
    subplot[subplot_id].yaxis.grid(which='major', color=major_grid_color, linestyle='-', linewidth=major_grid_width, zorder=102)
    subplot[subplot_id].set_ylim(bottom=y_min_view, top=y_max_view)

    x0 = 1
    for i_generation in range(0, n_generation):
        n_data = len(data_dict[subplot_id][i_generation])
        x_data = x0 + numpy.zeros(n_data)
        subplot[subplot_id].plot(x_data, data_dict[subplot_id][i_generation], linestyle="", marker="o", markersize=15.0, markeredgewidth=1.0, markeredgecolor='none', alpha=0.5, zorder=122)
        subplot[subplot_id].plot(x_data, data_dict[subplot_id][i_generation], linestyle="", marker="o", markersize=15.0, markeredgewidth=1.0, markeredgecolor='black', markerfacecolor='none', zorder=123)
        x0 = x0 + 1
    #
#

matplotlib.pyplot.savefig(plot_file_path)

matplotlib.pyplot.close(figure)

########################################################################################################################
exit()


